==> soccerapi-0.3.1.tar.gz: soccerapi-0.3.1/soccerapi/api/base.py <==
import abc
import json
import os
from typing import Dict, List, Tuple

import requests


class ApiBase(abc.ABC):
    """ The Abstract Base Class on which every Api[Boolmaker] is based on. """

    def _read_table(self) -> Dict:
        """ Read json table that store the relation between standard names and
        bookmaker names, ids and other. File name is {bookmaker}.json """

        here = os.path.dirname(__file__)
        table_path = os.path.join(here, self.name, f'{self.name}.json')
        with open(table_path) as f:
            table = json.load(f)
        return table

    def _country_league(self, country: str, league: str) -> Tuple[str, str]:
        """ Get standard country-league and convert to bookmaker country-league
        using self.table """

        try:
            country = self.table[country]['name']
        except KeyError:
            msg = (
                f'{country} is not in {self.name} table. '
                'Check the docs for a list of supported countries.'
            )
            raise KeyError(msg)
        try:
            league = self.table[country]['leagues'][league]['name']
        except KeyError:
            msg = (
                f'{league} is not in {self.name} table. '
                'Check the docs for a list of supported leagues.'
            )
            raise KeyError(msg)
        return country, league

    @abc.abstractmethod
    def odds(self, country: str, league: str, market: str = 'IT') -> Dict:
        """ Get the odds from the country-league competiton as a python dict """
        pass


class ApiKambi(ApiBase):
    """ 888sport, unibet and other use the same CDN (eu-offering.kambicdn)
     so the requetsting and parsing process is exaclty the same.
     The only thing that chage is the base_url """

    @staticmethod
    def _full_time_result(data: Dict) -> List:
        """ Parse the raw json requests for full_time_result """

        odds = []
        for event in data['events']:
            if event['event']['state'] == 'STARTED':
                continue
            try:
                full_time_result = {
                    '1': event['betOffers'][0]['outcomes'][0].get('odds'),
                    'X': event['betOffers'][0]['outcomes'][1].get('odds'),
                    '2': event['betOffers'][0]['outcomes'][2].get('odds'),
                }
            except IndexError:
                full_time_result = None

            odds.append(
                {
                    'time': event['event']['start'],
                    'home_team': event['event']['homeName'],
                    'away_team': event['event']['awayName'],
                    'full_time_resut': full_time_result,
                }
            )
        return odds

    @staticmethod
    def _both_teams_to_score(data: Dict) -> List:
        """ Parse the raw json requests for both_teams_to_score """

        odds = []
        for event in data['events']:
            if event['event']['state'] == 'STARTED':
                continue
            try:
                both_teams_to_score = {
                    'yes': event['betOffers'][0]['outcomes'][0].get('odds'),
                    'no': event['betOffers'][0]['outcomes'][1].get('odds'),
                }
            except IndexError:
                both_teams_to_score = None
            odds.append(
                {
                    'time': event['event']['start'],
                    'home_team': event['event']['homeName'],
                    'away_team': event['event']['awayName'],
                    'both_teams_to_score': both_teams_to_score,
                }
            )
        return odds

    @staticmethod
    def _double_chance(data: Dict) -> List:
        """ Parse the raw json requests for double chance """

        odds = []
        for event in data['events']:
            if event['event']['state'] == 'STARTED':
                continue
            try:
                double_chance = {
                    '1X': event['betOffers'][0]['outcomes'][0].get('odds'),
                    '12': event['betOffers'][0]['outcomes'][1].get('odds'),
                    '2X': event['betOffers'][0]['outcomes'][2].get('odds'),
                }
            except IndexError:
                double_chance = None
            odds.append(
                {
                    'time': event['event']['start'],
                    'home_team': event['event']['homeName'],
                    'away_team': event['event']['awayName'],
                    'double_chance': double_chance,
                }
            )
        return odds

    def _requests(self, country: str, league: str, market: str = 'IT') -> Tuple[Dict]:
        """ Build URL starting from country and league and request data for
            - full_time_result
            - both_teams_to_score
            - double_chance
        """

        s = requests.Session()
        base_params = {'lang': 'en_US', 'market': market}
        url = '/'.join([self.base_url, country, league]) + '.json'

        return (
            # full_time_result
            s.get(url, params={**base_params, 'category': 12579}).json(),
            # both_teams_to_score
            s.get(url, params={**base_params, 'category': 11942}).json(),
            # double_chance
            s.get(url, params={**base_params, 'category': 12220}).json(),
        )

    def odds(self, country: str, league: str, market: str = 'IT') -> Dict:
        """ Get odds from country-league competition """

        # Convert to standard country - league names
        country, league = self._country_league(country, league)

        # reuquest odds data
        odds = self._requests(country, league, market)

        # parse json response
        odds = [
            self._full_time_result(odds[0]),
            self._both_teams_to_score(odds[1]),
            self._double_chance(odds[2]),
        ]
        return [{**i, **j, **k} for i, j, k in zip(*odds)]

==> soccerapi-0.4.0.tar.gz: soccerapi-0.4.0/soccerapi/api/base.py <==
import abc
import csv
import json
import os
from typing import Dict, List, Tuple

import requests


class ApiBase(abc.ABC):
    """ The Abstract Base Class on which every Api[Boolmaker] is based on. """

    def _load_competitions(self) -> Dict:
        """ Read .csv from S1M0N38/soccerapi-competitions and create a
        dictioary of available competitions (not supported league are leave empty '')
        e.g. {'england-premier_league': '',
         'england-championship': 'E42294894',
         'germany-bundesliga_2': 'E42422121'}
        """
        competitions = {}
        url = (
            'https://raw.githubusercontent.com/'
            'S1M0N38/soccerapi-competitions/master/competitions.csv'
        )
        data = requests.get(url).text.splitlines()
        rows = csv.DictReader(data)
        for row in rows:
            key = f'{row["country"]}-{row["league"]}'
            competitions[key] = row[self.name]
        return competitions

    def _competition(self, country: str, league: str) -> str:
        """ Get standard country and league and return the corresponding
        competition id. Could be something like 'E42294894' (bet365) or
        'england/premier_league' (888sport, unibet)."""

        competition = f'{country}-{league}'
        msg = (
            f'{competition} is not supported for {self.name}. '
            'Check the docs for a list of supported competitions.'
        )
        try:
            competition_id = self.competitions[competition]
        except KeyError:
            raise KeyError(msg)
        if competition_id == '':
            raise KeyError(msg)
        return competition_id

    @abc.abstractmethod
    def odds(self, country: str, league: str, market: str = 'IT') -> Dict:
        """ Get the odds from the country-league competition as a python dict """
        pass


class ApiKambi(ApiBase):
    """ 888sport, unibet and other use the same CDN (eu-offering.kambicdn)
     so the requetsting and parsing process is exaclty the same.
     The only thing that chage is the base_url """

    @staticmethod
    def _full_time_result(data: Dict) -> List:
        """ Parse the raw json requests for full_time_result """

        odds = []
        for event in data['events']:
            if event['event']['state'] == 'STARTED':
                continue
            try:
                full_time_result = {
                    '1': event['betOffers'][0]['outcomes'][0].get('odds'),
                    'X': event['betOffers'][0]['outcomes'][1].get('odds'),
                    '2': event['betOffers'][0]['outcomes'][2].get('odds'),
                }
            except IndexError:
                full_time_result = None

            odds.append(
                {
                    'time': event['event']['start'],
                    'home_team': event['event']['homeName'],
                    'away_team': event['event']['awayName'],
                    'full_time_resut': full_time_result,
                }
            )
        return odds

    @staticmethod
    def _both_teams_to_score(data: Dict) -> List:
        """ Parse the raw json requests for both_teams_to_score """

        odds = []
        for event in data['events']:
            if event['event']['state'] == 'STARTED':
                continue
            try:
                both_teams_to_score = {
                    'yes': event['betOffers'][0]['outcomes'][0].get('odds'),
                    'no': event['betOffers'][0]['outcomes'][1].get('odds'),
                }
            except IndexError:
                both_teams_to_score = None
            odds.append(
                {
                    'time': event['event']['start'],
                    'home_team': event['event']['homeName'],
                    'away_team': event['event']['awayName'],
                    'both_teams_to_score': both_teams_to_score,
                }
            )
        return odds

    @staticmethod
    def _double_chance(data: Dict) -> List:
        """ Parse the raw json requests for double chance """

        odds = []
        for event in data['events']:
            if event['event']['state'] == 'STARTED':
                continue
            try:
                double_chance = {
                    '1X': event['betOffers'][0]['outcomes'][0].get('odds'),
                    '12': event['betOffers'][0]['outcomes'][1].get('odds'),
                    '2X': event['betOffers'][0]['outcomes'][2].get('odds'),
                }
            except IndexError:
                double_chance = None
            odds.append(
                {
                    'time': event['event']['start'],
                    'home_team': event['event']['homeName'],
                    'away_team': event['event']['awayName'],
                    'double_chance': double_chance,
                }
            )
        return odds

    def _requests(self, competition: str, market: str = 'IT') -> Tuple[Dict]:
        """ Build URL starting from country and league and request data for
            - full_time_result
            - both_teams_to_score
            - double_chance
        """
        s = requests.Session()
        base_params = {'lang': 'en_US', 'market': market}
        url = '/'.join([self.base_url, competition]) + '.json'

        return (
            # full_time_result
            s.get(url, params={**base_params, 'category': 12579}).json(),
            # both_teams_to_score
            s.get(url, params={**base_params, 'category': 11942}).json(),
            # double_chance
            s.get(url, params={**base_params, 'category': 12220}).json(),
        )

    def odds(self, country: str, league: str, market: str = 'IT') -> Dict:
        """ Get odds from country-league competition """

        # get competition id for country-league
        competition = self._competition(country, league)

        # reuquest odds data
        odds = self._requests(competition, market)

        # parse json response
        odds = [
            self._full_time_result(odds[0]),
            self._both_teams_to_score(odds[1]),
            self._double_chance(odds[2]),
        ]
        return [{**i, **j, **k} for i, j, k in zip(*odds)]
